<?php

namespace Drupal\private_message\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\private_message\Ajax\PrivateMessageInboxUpdateCommand;
use Drupal\private_message\Ajax\PrivateMessageInsertNewMessagesCommand;
use Drupal\private_message\Ajax\PrivateMessageInsertThreadsCommand;
use Drupal\private_message\Ajax\PrivateMessageInsertPreviousMessagesCommand;
use Drupal\private_message\Ajax\PrivateMessageMembersAutocompleteResponseCommand;
use Drupal\private_message\Service\PrivateMessageServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

class AjaxController extends ControllerBase implements AjaxControllerInterface  {

  const AUTOCOMPLETE_COUNT = 10;

  /**
   * The renderer service
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The request stack
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The entity manager service
   *
   * @param \Drupal\Core\Entity\EntityManagerInterface
   */
  protected $entityManager;

  /**
   * The configuration factory
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The current user
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The private message service
   *
   * @var \Drupal\private_message\Service\PrivateMessageServiceInterface
   */
  protected $privateMessageService;

  /**
   * Constructs n AjaxController object
   *
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack
   * @param \Drupal\Core\Entity\EntityManagerInterface $entityManager
   *   The entity manager service
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration factory
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user
   * @param \Drupal\private_message\Service\PrivateMessageServiceInterface $privateMessageService
   *   The private message service
   */
  public function __construct(RendererInterface $renderer, RequestStack $requestStack, EntityManagerInterface $entityManager, ConfigFactoryInterface $configFactory, AccountProxyInterface $currentUser, PrivateMessageServiceInterface $privateMessageService) {
    $this->renderer = $renderer;
    $this->requestStack = $requestStack;
    $this->entityManager = $entityManager;
    $this->configFactory = $configFactory;
    $this->currentUser = $currentUser;
    $this->privateMessageService = $privateMessageService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('renderer'),
      $container->get('request_stack'),
      $container->get('entity.manager'),
      $container->get('config.factory'),
      $container->get('current_user'),
      $container->get('private_message.service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function ajaxCallback($op) {
    $response = new AjaxResponse();

    if ($this->currentUser->hasPermission('use private messaging system')) {
      switch($op) {
        case 'get_new_messages': {
          $thread_id = $this->requestStack->getCurrentRequest()->get('threadid');
          $message_id = $this->requestStack->getCurrentRequest()->get('messageid');
          if (is_numeric($thread_id) && is_numeric($message_id)) {
            $new_messages = $this->privateMessageService->getNewMessages($thread_id, $message_id);
            if (count($new_messages)) {
              $messages = [];
              $view_builder = $this->entityManager->getViewBuilder('private_message');
              foreach ($new_messages as $message) {
                $messages[] = $view_builder->view($message);
              }

              $command = new PrivateMessageInsertNewMessagesCommand($this->renderer->renderRoot($messages));
              $response->addCommand($command);
            }
          }

          break;
        }

        case 'get_old_messages': {
          $current_request = $this->requestStack->getCurrentRequest();
          $thread_id = $current_request->get('threadid');
          $message_id = $current_request->get('messageid');
          $message_count = $current_request->get('count');
          if (is_numeric($thread_id) && is_numeric($message_id)) {
            $message_info = $this->privateMessageService->getPreviousMessages($thread_id, $message_id);

            if (count($message_info)) {
              $messages = [];
              $view_builder = $this->entityManager->getViewBuilder('private_message');
              foreach ($message_info as $message) {
                $messages[] = $view_builder->view($message);
              }

              $command = new PrivateMessageInsertPreviousMessagesCommand($this->renderer->renderRoot($messages));
            }
            else {
              $command = new PrivateMessageInsertPreviousMessagesCommand('');
            }

            $response->addCommand($command);
          }

          break;
        }

        case 'get_old_inbox_threads': {
          $timestamp = $this->requestStack->getCurrentRequest()->get('timestamp');
          $thread_count = $this->requestStack->getCurrentRequest()->get('count');
          if (is_numeric($timestamp)) {
            $thread_info = $this->privateMessageService->getThreadsForUser($thread_count, $timestamp);
            if (count($thread_info['threads'])) {
              $oldest_timestamp = FALSE;
              $view_builder = $this->entityManager->getViewBuilder('private_message_thread');
              $threads = [];
              foreach ($thread_info['threads'] as $thread) {
                if ($thread_info['next_exists']) {
                  $oldest_timestamp = $thread->get('updated')->value;
                }
                $threads[] = $view_builder->view($thread, 'inbox');
              }

              $command = new PrivateMessageInsertThreadsCommand($this->renderer->renderRoot($threads), $oldest_timestamp);
            }
            else {
              $command = new PrivateMessageInsertThreadsCommand('', FALSE);
            }

            $response->addCommand($command);
          }

          break;
        }

        case 'get_new_inbox_threads': {
          $info = $this->requestStack->getCurrentRequest()->get('ids');

          // Check to see if any thread IDs were POSTed
          if (count($info)) {
            // Get new inbox information based on the posted IDs
            $inbox_threads = $this->privateMessageService->getUpdatedInboxThreads($info);
          }
          else {
            // No IDs were posted, so the maximum possible number of threads to be returned is
            // retrieved from the block settings.
            $thread_count = $this->configFactory->get('block.block.privatemessageinbox')->get('settings.thread_count');
            $inbox_threads = $this->privateMessageService->getUpdatedInboxThreads([], $thread_count);
          }

          // Only need to do something if any thread IDS were found
          if (count($inbox_threads['thread_ids'])) {
            $view_builder = $this->entityManager->getViewBuilder('private_message_thread');

            // Render any new threads as HTML to be sent to the browser.
            $rendered_threads = [];
            foreach (array_keys($inbox_threads['new_threads']) as $thread_id) {
              $renderable = $view_builder->view($inbox_threads['new_threads'][$thread_id], 'inbox');
              $rendered_threads[$thread_id] = $this->renderer->renderRoot($renderable);
            }

            // Add the command that will tell the inbox which thread items to update
            $response->addCommand(new PrivateMessageInboxUpdateCommand($inbox_threads['thread_ids'], $rendered_threads));
          }

          break;
        }
      }
    }

    return $response;
  }

  public function privateMessageMembersAutocomplete() {
    $response = new AjaxResponse();

    $username = $this->requestStack->getCurrentRequest()->get('username');
    $accounts = $this->privateMessageService->getUsersFromString($username, self::AUTOCOMPLETE_COUNT);
    $user_info = [];
    foreach ($accounts as $account) {
      $user_info[] = [
	      'uid' => $account->id(),
	      'username' => $account->getDisplayName(),
      ];
    }

    $response->addCommand(new PrivateMessageMembersAutocompleteResponseCommand($username, $user_info));

    return $response;
  }
}
